# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-12-17 13:01
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_project', '0004_auto_20191211_1552'),
    ]

    operations = [
        migrations.RenameField(
            model_name='customer',
            old_name='firstName',
            new_name='first_name',
        ),
        migrations.RenameField(
            model_name='customer',
            old_name='lastName',
            new_name='last_name',
        ),
    ]



from djangoldp.permissions import LDPPermissions
from django.db.models.base import ModelBase
from djangoldp_project.xmpp import XMPP_SERVERS, get_client_ip
from djangoldp_project.filters import CustomerFilterBackend, ProjectFilterBackend, ProjectMemberFilterBackend


class CustomerPermissions(LDPPermissions):
    filter_backends=[CustomerFilterBackend]

    def user_permissions(self, user, obj_or_model, obj=None):
        from .models import Member

        if not isinstance(obj_or_model, ModelBase):
            obj = obj_or_model

        # start with the permissions set on the object and model
        perms = set(super().user_permissions(user, obj_or_model, obj))

        if not user.is_anonymous:
            # object-level permissions
            if obj and not isinstance(obj, ModelBase):
                # members of one of their projects can view the customer
                if Member.objects.filter(project__customer=obj, user=user).exists():
                    perms.add('view')

            # model-level permissions
            else:
                perms = perms.union({'view', 'add'})

        return list(perms)


class ProjectPermissions(LDPPermissions):
    filter_backends=[ProjectFilterBackend]

    def user_permissions(self, user, obj_or_model, obj=None):
        if not isinstance(obj_or_model, ModelBase):
            obj = obj_or_model

        # start with the permissions set on the object and model
        perms = set(super().user_permissions(user, obj_or_model, obj))

        if not user.is_anonymous:
            # object-level permissions
            if obj and not isinstance(obj, ModelBase):
                # permissions gained by being a member or an admin
                if obj.members.filter(user=user).exists():
                    perms.add('view')

                    if obj.members.filter(user=user).get().is_admin:
                        perms = perms.union({'add', 'change', 'delete'})

                # permissions gained by the project being public
                if obj.status == 'Public':
                    perms = perms.union({'view', 'add'})

            # model-level permissions
            else:
                perms = perms.union({'view', 'add'})

        return list(perms)
    
    def has_permission(self, request, view):
        if get_client_ip(request) in XMPP_SERVERS:
            return True

        return super().has_permission(request, view)
    
    def has_object_permission(self, request, view, obj):
        if get_client_ip(request) in XMPP_SERVERS:
            return True

        return super().has_object_permission(request, view, obj)


class ProjectMemberPermissions(LDPPermissions):
    filter_backends = [ProjectMemberFilterBackend]

    def user_permissions(self, user, obj_or_model, obj=None):
        if not isinstance(obj_or_model, ModelBase):
            obj = obj_or_model

        # start with the permissions set on the object and model
        perms = set(super().user_permissions(user, obj_or_model, obj))

        if not user.is_anonymous:
            # object-level permissions
            if obj and hasattr(obj, 'user') and not isinstance(obj, ModelBase):
                # the operation is on myself
                if obj.user == user:
                    perms.add('view')

                    if not obj.is_admin or obj.project.members.filter(is_admin=True).count() > 1:
                        perms.add('delete')

                    if obj.project.status == 'Public':
                        perms = perms.union({'add', 'delete'})

                # the operation is on another user
                else:
                    # permissions gained in public circles
                    if obj.project.status == 'Public':
                        perms = perms.union({'view', 'add'})

                    # permissions gained for membership
                    if obj.project.members.filter(user=user).exists():
                        if obj.project.members.filter(user=user).get().is_admin:
                            perms = perms.union({'view', 'add'})

                            if not obj.is_admin:
                                perms.add('delete')
                        else:
                            perms.add('view')

            # model-level permissions
            else:
                perms = perms.union({'view', 'add'})

        return list(perms)
    
    def has_permission(self, request, view):
        if get_client_ip(request) in XMPP_SERVERS:
            return True

        return super().has_permission(request, view)
    
    def has_object_permission(self, request, view, obj):
        if get_client_ip(request) in XMPP_SERVERS:
            return True

        return super().has_object_permission(request, view, obj)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-09-03 10:51
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_project', '0013_auto_20200619_0808'),
    ]

    operations = [
        migrations.AlterField(
            model_name='businessprovider',
            name='fee',
            field=models.PositiveIntegerField(blank=True, default='0', null=True),
        ),
        migrations.AlterField(
            model_name='businessprovider',
            name='name',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='customer',
            name='name',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='member',
            name='project',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='members', to='djangoldp_project.Project'),
        ),
        migrations.AlterField(
            model_name='member',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='projects', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='project',
            name='customer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='djangoldp_project.Customer'),
        ),
        migrations.AlterField(
            model_name='project',
            name='name',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='project',
            name='status',
            field=models.CharField(blank=True, choices=[('Public', 'Public'), ('Private', 'Private'), ('Archived', 'Archived')], default='Private', max_length=8, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-06-01 15:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_project', '0010_auto_20200522_1516'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='member',
            name='name',
        ),
    ]



# Generated by Django 2.2.16 on 2020-11-30 14:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_project', '0015_auto_20200926_0911'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='project',
            name='team',
        ),
    ]
